{
  RASManager.pas

  Written by Frank Plagge
  Copyright (c) 1998 by Frank Plagge, Elsterweg 39, 38446 Wolfsburg, Germany
  All rights reserved

  *****************************************************************************
  Based on:

  ras.pas - Copyright by Davide Moretti <dmoretti@iper.net>

  Remote Access external API
  Public header for external API clients
  ras.h   - Copyright (c) 1992-1996, Microsoft Corporation, all rights reserved
  *****************************************************************************

  Please send comments to plagge@positiv.escape.de

  V 1.01 - Aug 7th, 1998
           first implementation, never trust a version 1.00 :-)
  V 1.02 - Aug 23th, 1998
           + added real connected state
             the property Connected represents the connection state from
             RAS services. if the connection state for a service is needed
             simply set the property RASName and next check the poperty
             Connected.
  V 1.03 - Sep 29th, 1998
           + fixed some uninititialized function results
  V 1.04 - Oct 17th, 1998
           + minor changes
           + documentation completed - it's never complete :-)
           + first public release

  *****************************************************************************
  NO REPRESENTATIONS ARE MADE ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
  PURPOSE. IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
  NEITHER FRANK PLAGGE OR ANY OTHER PERSON SHALL BE LIABLE FOR ANY DAMAGES
  SUFFERED BY THE USE OF THIS SOFTWARE.
  *****************************************************************************

  description:
   This module contains the component TRASManager. It contains a component
   wrapper for RAS access. It is possible to connect and disconnect a RAS
   service very easy. A connection timeout is integrated. this component is
   ready to use for easy functions, I know there are much more possibilities
   with RAS, but there is no time to implement all of them.

  *****************************************************************************

  properties and events at design time:

   property DialAsync: Boolean
   with RAS services asynchronous dialing is possible. setting this property
   true will enable the asynchronous dialing. only if this is enabled, the
   OnConnecting event is gererated.
   CAUTION: for the moment is is not possible to do a asynchronous dialing for
            more than one RAS object at the same time! perhaps I will extend
            this in the future.


   property Timeout: Longint
   the RAS Manager has an internal time for a connection timeout. is this value
   is unequal to zero the timeout mechanism is enalbed if a connection will be
   establied via the method Connect. if the time occures the connection is NOT
   closed automaticly. a timeout will ONLY cause the OnTimeout event. the timeout
   mechanism works NOT if the service was connected by another program; perhaps
   I will change this in the future.

   property OnConnecting: TOnConnectingEvent
   this event is caused by the asynchronous dialing

   property OnError: TOnErrorEvent
   this event is called if an error occures during connection or during
   finding the dialing parameters

   property OnTimeout: TNotifyEvent
   this event is called if a connection timeout occures. the connection is NOT
   closed!

  *****************************************************************************
   properties and events at runtime:

   procedure Connect
   try to connect the RAS service

   procedure DisConnect
   try to disconnect a formerly connected RAS service. this works also if the
   the named service was connect by another program

   property Connected: Boolean -READONLY-
   determine if the RAS service is already connected. this works also if the
   given service was connected by another program

   property Password: string -READONLY-
   get the password of the RAS service

   property RASName: string
   set the name of the required RAS connection

   property Time: Longint -READONLY-
   get the connection time of the RAS service. this works NOT if the service
   was connected by another program

   property Username: string -READONLY-
   get the username of the RAS service

   property  Version: string -READONLY-
   get the internal version number. maybe this will be helpful in the furture.
   I don't know now


  *****************************************************************************
  other very useful procedures

  procedure GetRASConnected( Names: TStrings );
  get the names of all already connected RAS services

  procedure GetRASNames( Names: TStrings );
  get the names of all possible RAS services

  function  IsRASConnected( const RASName: string ): Boolean;
  determine if a RAS connection with the NAme RASName is available

}

{$A+} // word alignment

unit RASManager;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, RAS;

const
  cVersion      = '1.04';   // internal version number
  cRegisterName = 'TCP/IP'; // name for component registration

type

  // error event type, the Error value contains the error code, look
  // into ras.pas for more informations
  TOnErrorEvent = procedure (Sender: TObject; Error: Integer) of object;

  // asynchronous dialing event
  TOnConnectingEvent = procedure (Sender: TObject; Msg: Integer; State: Integer; Error: Longint) of object;

  // the RASManager Component itself
  TRASManager = class(TComponent)
  private
    FDialAsync: Boolean;                // asynchronous dialing enabled?
    FOnConnecting: TOnConnectingEvent;  // event for asynchronous dialing
    FOnError:      TOnErrorEvent;       // error event
    FOnTimeout:    TNotifyEvent;        // timeout event
    FRASHandle:   THRasConn;            // connection handle
    FRASName:     string;               // name of the RAS service
    FTimeout:     Longint;              // timeout value (seconds)
    RASSeconds:   LongInt;              // online time (seconds)
    RASTimer:     TTimer;               // internal timer for timeout
    function  GetConnected: Boolean;
    function  GetParams(Server: string; var DialParams: TRasDialParams): Boolean;
    function  GetPassword: string;
    procedure GetRASHandle;
    function  GetUsername: string;
    function  GetVersion: string;
    procedure SetRASName( Value: string );
    procedure SetTimeout( Value: Longint );
    procedure OnTimer( Sender: TObject );
  public
    constructor Create( AOwner: TComponent ); override; // create the object
    destructor  Destroy; override;                      // and destroy it
    procedure   Connect;                                // make a connection
    procedure   DisConnect;                             // close the connection
    property    Connected: Boolean read GetConnected;   // is service connected?
    property    Password: string read GetPassword;      // get the password
    property    RASName: string read FRASName write SetRASName; // name of RAS service
    property    Time: Longint read RASSeconds;          // online time
    property    Username: string read GetUsername;      // username
    property    Version: string read GetVersion;        // version of this component
  published
    property DialAsync: Boolean read FDialAsync write FDialAsync;  // asynch dialing
    property Timeout: Longint read FTimeout write SetTimeout;      // timeout value
    property OnConnecting: TOnConnectingEvent read FOnConnecting write FOnConnecting; // asynch dialing event
    property OnError: TOnErrorEvent read FOnError write FOnError;                     // error event
    property OnTimeout: TNotifyEvent read FOnTimeout write FOnTimeout;                // timeout event
  end;

procedure GetRASConnected( Names: TStrings );               // get all existing connections
procedure GetRASNames( Names: TStrings );                   // get all possible connections
function  IsRASConnected( const RASName: string ): Boolean; // test if a connection is available

procedure Register;

implementation

var RASSave: TRASManager;

// ***************************************************************************
// callback function if asynchronous dialing is enabled
procedure RASCallback(Msg: Integer; State: TRasConnState; Error: Longint); stdcall
begin
  // if an asnych event is defined the event is called
  if Assigned(RASSave.FOnConnecting) then begin
    RASSave.FOnConnecting( RASSave, Msg, State, Error );
  end;
  // if an error event is defined and an error occurred then
  // the error event is called
  if (Assigned(RASSave.FOnError)) and (Error<>0) then begin
    RASSave.FOnError( RASSave, Error );
  end;
end;

// ***************************************************************************
// create a RASManager object
constructor TRASManager.Create( AOwner: TComponent );
begin
  inherited Create(AOwner);           // create the component first
  FDialAsync := false;                // no default async dialing
  FRASHandle := 0;                    // internal RAS handle
  FRASName := '';                     // no default RAS name
  FTimeout := 0;                      // no default timeout
  RASTimer := TTimer.Create( Self );  // create an internal timer
  RASTimer.Enabled := false;          // timer is not enabled
  RASTimer.OnTimer := OnTimer;        // set the timer event
  RASTimer.Interval := 1000;          // a one second interval is needed
end;

// ***************************************************************************
// destroy a formerly created RASManager object
destructor TRASManager.Destroy;
begin
  RASTimer.Free;        // first free the RAS timer
  inherited Destroy;    // next destroy the object
end;

// ***************************************************************************
// connect to the RAS service specified in the property RASName
procedure TRASManager.Connect;
var DialParams: TRasDialParams;  // local dial parameters
    RASResult:  LongInt;         // used result for RASDial call, not used
begin
  if not Connected then begin                       // only if the service is not connected
    if GetParams( FRASName, DialParams ) then begin // get actual dial parameters
      RASSave := self;                              // save the object itself
      if FDialAsync then begin                      // if async dialing is enabled
        RASResult := RasDial(nil, nil, DialParams, 0, @RASCallback, FRASHandle ); // call with a callback function
      end else begin                                // if async dialing is disabled
        RASResult := RasDial(nil, nil, DialParams, 0, nil, FRASHandle );          // call without callback function
      end;
      // if the service is not connected it may be necessary to disconnect
      if not Connected then begin          // if no connection is established
        if FRASHandle<>0 then begin        // only if a vaild handle is available
          DisConnect;                      // disconnect
        end;
        if (Assigned(FOnError)) then begin // if an error event is assigned
          FOnError( self, RASResult );     // call the error event
        end;
      end;
    end;
    if Connected then begin                 // if a connection is established
      RASSeconds := 0;                      // reset the online time counter
      RASTimer.Enabled := true;             // enabled the online timer
    end;
  end;
end;

// ***************************************************************************
// disconnect the already connected RAS service
procedure TRASManager.DisConnect;
begin
  if Connected then begin              // only if a connection is available
    if FRASHandle<>0 then begin        // only if a vaild handle is available
      RasHangup( FRASHandle );         // hangup the RAS service
    end;
    RASTimer.Enabled := false;         // disable the online counter
  end;
end;

// ***************************************************************************
// read method for property Connected
// determine if the service is connected
function TRASManager.GetConnected: Boolean;
begin
  Result := IsRASConnected( FRASName );      // test if a service with this name is established
  if (Result) and (FRASHandle=0) then begin  // if no handle is available
    GetRASHandle;                            // try to read the handle
  end;
end;

// ***************************************************************************
// get the dial parameters for a ras server
function TRASManager.GetParams(Server: string; var DialParams: TRasDialParams): Boolean;
var DialPassword: LongBool;
    RASResult:    LongInt;
begin
  Result := true;                                     // result is first vaild
  FillChar( DialParams, SizeOf(TRasDialParams), 0);   // clear the result record
  DialParams.dwSize := Sizeof(TRasDialParams);        // set the result array size
  StrPCopy(DialParams.szEntryName, Server);           // set the ras service name
  DialPassword := true;                               // get the dial password
  RASResult := RasGetEntryDialParams(nil, DialParams, DialPassword); // read the ras parameters
  if (RASResult<>0) then begin                        // if the API call was not successful
    Result := false;                                  // result is not vaild
    if (Assigned(FOnError)) then begin                // if an error event is assigned
      FOnError( self, RASResult );                    // call the error event
    end;
  end;
end;

// ***************************************************************************
// read method for property Password
function TRASManager.GetPassword: string;
var DialParams: TRasDialParams;                   // dial parameters for this service
begin
  if GetParams( FRASName, DialParams ) then begin // if read of dial parameters was successful
    Result := DialParams.szPassword;              // copy the password string
  end else begin                                  // if read was not successful
    Result := '';                                 // return an empty string
  end;
end;

// ***************************************************************************
// try to determine a handle for the given RASName
// if this is successfull, an existing RAS connection can be controled
// by this object
procedure TRASManager.GetRASHandle;
const cMaxRas = 100;                           // maximum number of ras services
var BufferSize: LongInt;                       // used for size of result buffer
    RASBuffer:  array[1..cMaxRas] of TRasConn; // the API result buffer itself
    RASCount:   LongInt;                       // number of found ras services
    i:          Integer;                       // loop counter
begin
  FRASHandle := 0;                             // first no handle is available
  FillChar( RASBuffer, SizeOf(RASBuffer), 0 ); // clear the API Buffer
  RASBuffer[1].dwSize := SizeOf(TRasConn);     // set the API buffer size for a single record
  BufferSize := SizeOf(TRasConn) * cMaxRas;    // calc complete buffer size
  // if the API call causes no error
  if RasEnumConnections(@RASBuffer[1], BufferSize, RASCount) = 0 then begin
    for i := 1 to RASCount do begin            // for all found ras services
      if RASBuffer[i].szEntryName = RASName then begin // if the actual name is available
        FRASHandle := RASBuffer[i].hrasconn;   // save the found ras handle
      end;
    end;
  end;
end;

// ***************************************************************************
// read method for property Username
function TRASManager.GetUsername: string;
var DialParams: TRasDialParams;                   // dial parameters for this service
begin
  if GetParams( FRASName, DialParams ) then begin // if read of dial parameters was successful
    Result := DialParams.szUserName;              // copy the user name string
  end else begin                                  // if read was not successful
    Result := '';                                 // return an empty string
  end;
end;

// ***************************************************************************
// read method for property Version
function TRASManager.GetVersion: string;
begin
  Result := cVersion;  // return the internal version number
end;

// ***************************************************************************
// handle the event for the internal timer; timer base is 1 second
// CAUTION: if a timeout occures the ras service will NOT disconnect!
//          the owner of this service must disconnect the service!
procedure TRASManager.OnTimer( Sender: TObject );
begin
  Inc( RASSeconds, 1 );                  // increment online timer
  if (Timeout>0) and (RASSeconds>=Timeout) then begin // if the timeout is reached
    Timeout := 0;                        // disable timeout
    if Assigned(FOnTimeout) then begin   // if a timeout event if defined
      FOnTimeout( self );                // call the timeout event
    end;
  end;
end;

// ***************************************************************************
// write method for property RASName
procedure TRASManager.SetRASName( Value: string );
var DialParams: TRasDialParams;                   // dial parameters for this service
begin
  // only if its is possible to read the dial parameters for the ras service
  // name the name is accepted
  if GetParams( Value, DialParams ) then begin
    FRASName := Value;
    GetRASHandle;        // try to read an existing handle
  end;
end;

// ***************************************************************************
// write method for property Timeout
procedure TRASManager.SetTimeout( Value: Longint );
begin
  // if the timeout value is less than zero it is not valid
  // a message box appers, because this is an important user error
  if Value < 0 then begin
    MessageDlg( 'RASManager: Invalid Timeout value (0..MaxInt)', mtError, [mbOk], 0 );
  end else begin
    FTimeout := Value;
  end;
end;

// ***************************************************************************
// get the names of all connected ras services and return the names within
// a TStrings object
// CAUTION: the result object must exist!
procedure GetRASConnected( Names: TStrings );
const cMaxRas = 100;                           // maximum number of ras services
var BufferSize: LongInt;                       // used for size of result buffer
    RASBuffer:  array[1..cMaxRas] of TRasConn; // the API result buffer itself
    RASCount:   LongInt;                       // number of found ras services
    i:          Integer;                       // loop counter
begin
  Names.Clear;                                 // clear the name list
  FillChar( RASBuffer, SizeOf(RASBuffer), 0 ); // clear the API Buffer
  RASBuffer[1].dwSize := SizeOf(TRasConn);     // set the API buffer size for a single record
  BufferSize := SizeOf(TRasConn) * cMaxRas;    // calc complete buffer size
  // if the API call causes no error
  if RasEnumConnections(@RASBuffer[1], BufferSize, RASCount) = 0 then begin
    for i := 1 to RASCount do begin            // for all found ras services
      Names.Add( RASBuffer[i].szEntryName );   // copy the name of the ras service
    end;
  end;
end;

// ***************************************************************************
// get the names of all available ras services and return the names within
// a TStrings object
// CAUTION: the result object must exist!
procedure GetRASNames( Names: TStrings );
const cMaxRas = 100;                            // maximum number of ras services
var BufferSize: LongInt;                        // used for size of result buffer
    RASBuffer:  array[1..cMaxRas] of TRasEntryName; // the API result buffer itself
    RASCount:   LongInt;                        // number of found ras services
    i:          Integer;                        // loop counter
begin
  Names.Clear;                                  // clear the name list
  FillChar( RASBuffer, SizeOf(RASBuffer), 0 );  // clear the API Buffer
  RASBuffer[1].dwSize := SizeOf(TRasEntryname); // set the API buffer size for a single record
  BufferSize := SizeOf(TRasEntryName) * cMaxRas;// calc complete buffer size
  // if the API call causes no error
  if RasEnumEntries(nil, nil, @RASBuffer[1], BufferSize, RASCount) = 0 then begin
    for i := 1 to RASCount do begin             // for all found ras services
      Names.Add( RASBuffer[i].szEntryName );    // copy the name of the ras service
    end;
  end;
end;

// ***************************************************************************
// check if a given ras name is connected
function IsRASConnected( const RASName: string ): Boolean;
var RASNames: TStrings;       // result object for connected services
    i:        Integer;        // loop counter
begin
  Result := false;                      // first the result is false
  RASNames := TStringList.Create;       // create the object for connected services
  GetRasConnected( RASNames );          // get all connected services
  for i := 1 to RASNames.Count do begin // for all connected services
    if RASName = RASNames.Strings[i-1] then begin // if the ras name was found
      Result := true;                             // the result is true now
      Break;                                      // break the loop, one is found
    end;
  end;
  RASNames.Free;                        // destroy the object for connected services
end;

// ***************************************************************************
// register the component
procedure Register;
begin
  RegisterComponents( cRegisterName, [TRASManager]);
end;

end.
